
import numpy as np

from sklearn.ensemble import AdaBoostClassifier
from sklearn.metrics import zero_one_loss
from sklearn.naive_bayes import MultinomialNB
from sklearn.externals import joblib
from sklearn.svm import SVC
import time


# 计算平均错误率
def cal_average(error):
    n = len(error)
    sum_error = 0
    average_error = 0.0
    i = 0

    for i in range(n):
        sum_error += error[i]
    average_error = sum_error/n
    return average_error


# 计算贝叶斯的分类错误个数
def Bayes_error(train_array, train_array_label, test_array, test_array_label):
    multi = MultinomialNB()
        #multi = BernoulliNB()
    multi = multi.fit(train_array, train_array_label)
        # 保存模型
    joblib.dump(multi, '../model/two_class_Bayes.pkl')

    # 加载模型
    start_time = time.clock()
    muljob = joblib.load('../model/two_class_Bayes.pkl')
    # 直接给出测试集的预测类别输出
    result = muljob.predict(test_array)
    end_time = time.clock()
    count = 0
    for i in range(len(result)):
        if result[i] != test_array_label[i]:
            count = count + 1
    return muljob, count, end_time - start_time


# 计算SVM的分类错误率
def SVM_error(train_array, train_array_label, test_array, test_array_label):
    clf = SVC(kernel='linear')
    clf.fit(np.array(train_array), np.array(train_array_label))
    joblib.dump(clf, '../model/two_class_SVM.pkl')

    start_time = time.clock()
    clf = joblib.load('../model/two_class_SVM.pkl')
    result = []
    result = clf.predict(test_array)

    num = 0
    for j in range(len(result)):
        if result[j] != test_array_label[j]:
            num = num + 1
    end_time = time.clock()
    return num, end_time - start_time


# 计算Adaboost的分类错误率
def Adaboost_error(multi, train_array, train_array_label, test_array, test_array_label):
    n_estimators = 500
    learning_rate = 1.

    start_time = time.clock()
    ada_real = AdaBoostClassifier(base_estimator=multi, learning_rate=learning_rate, n_estimators=n_estimators)
    # 拟合数据
    ada_real.fit(train_array, train_array_label)
    joblib.dump(ada_real, '../model/two_class_adaboost.pkl')

    test = []
    i = 0
    # 返回每个基分类器的预测数据集X的结果
    for i, y_pred in enumerate(ada_real.staged_predict(test_array)):
        # 默认情况下，返回的是所以样本上的损失的平均损失，把参数normalize设置为False，就可以返回损失值和
        # 在多标签分类问题中，如果预测的标签子集和真实的标签子集严格匹配，zero_one_loss函数给出得分为1，
        # 如果没有任何的误差，得分为0
        test.append(zero_one_loss(y_pred, test_array_label))
    end_time = time.clock()
    return ada_real, min(test), end_time - start_time