import re
import jieba
import numpy as np

jieba.load_userdict("../data/word.txt")
stop = [line.strip() for line in open('../data/stop.txt', 'r', encoding='utf-8').readlines()]  # 停用词


def build_key_word(path):
    d = {}
    with open(path, encoding="utf-8") as fp:
        for line in fp:
            for word in jieba.cut(line.strip()):
                p = re.compile(b'\w', re.L)  # 编译正则表达式，匹配字母数字及下划线，re.L表示特殊字符集
                result = p.sub(b"", bytes(word, encoding="utf-8")).decode("utf-8")
                #  空字符
                if not result or result == ' ':
                    continue
                # 避免大量无意义的词语进入统计范围
                if len(word) > 1:
                    d[word] = d.get(word, 0) + 1
    kw_list = sorted(d, key=lambda x: d[x], reverse=True)
    # 取最前的20%
    size = int(len(kw_list) * 0.2)
    mood = set(kw_list[:size])
    return list(mood - set(stop))


# 返回每条微博的分词与标签
def loadDataSet(path_or_text, mode):
    line_cut = []
    label = []
    if mode == "train" or mode == "test":
        with open(path_or_text, encoding="utf-8") as fp:
            for line in fp:
                temp = line.strip()
                try:
                    sentence = temp[2:].lstrip()  # 每条微博
                    first_label = int(temp[:2])
                    if first_label == 3:
                        continue
                    label.append(first_label)  # 获取标注
                    word_list = []
                    sentence = str(sentence).replace('\u200b', '')
                    for word in jieba.cut(sentence.strip()):
                        p = re.compile(r'\w', re.L)
                        result = p.sub("", word)
                        if not result or result == ' ':  # 空字符
                            continue
                        word_list.append(word)
                    word_list = list(set(word_list) - set(stop) - set('\u200b') - set(' ') - set('\u3000') - set('️'))
                    line_cut.append(word_list)
                except Exception:
                    continue
        return line_cut, label

    elif mode == "text":
        temp = path_or_text.strip()
        # 每条微博
        sentence = temp[2:].lstrip()
        word_list = []
        sentence = str(sentence).replace('\u200b', '')
        for word in jieba.cut(sentence.strip()):
            p = re.compile(r'\w', re.L)
            result = p.sub("", word)
            # 空字符
            if not result or result == ' ':
                continue
            word_list.append(word)
        word_list = set(word_list) - set(stop) - set('\u200b') - set(' ') - set('\u3000') - set('️')
        return word_list


# 每条微博向量化
def setOfWordsToVecTor(vocabularyList, moodWords):
    vocabMarked = [0] * len(vocabularyList)
    for smsWord in moodWords:
        if smsWord in vocabularyList:
            vocabMarked[vocabularyList.index(smsWord)] += 1
    return np.array(vocabMarked)


# 将所有微博准备向量化
def setOfWordsListToVecTor(vocabularyList, train_mood_array):
    vocabMarkedList = []
    for i in range(len(train_mood_array)):
        vocabMarked = setOfWordsToVecTor(vocabularyList, train_mood_array[i])
        vocabMarkedList.append(vocabMarked)
    return vocabMarkedList